/*
# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
*/

#ifndef CUT_TABLE_H
#define CUT_TABLE_H

int cut_table(const char* infn, const char* outfn, int N);

#endif

