/*
# This file is part of libkd.
# Licensed under a 3-clause BSD style license - see LICENSE
*/

typedef double etype;

#define ETYPE_INTEGER 0

#define ETYPE_MAX  KDT_LARGEVAL
#define ETYPE_MIN -KDT_LARGEVAL

#define ETYPE d
