/*
# This file is part of libkd.
# Licensed under a 3-clause BSD style license - see LICENSE
*/

typedef u64 etype;

#define ETYPE_INTEGER 1

#define ETYPE_MAX  UINT64_MAX
#define ETYPE_MIN  0

#define ETYPE l
