/*
# This file is part of the Astrometry.net suite.
# Licensed under a 3-clause BSD style license - see LICENSE
 */

#ifndef SCRIPTUTILS_H
#define SCRIPTUTILS_H

#include "astrometry/bl.h"

int parse_depth_string(il* depths, const char* str);

#endif
